#pragma once

/* Types */
typedef volatile unsigned int mthread_tst_t;

struct mthread_list_s;
typedef struct mthread_list_s mthread_list_t;

struct mthread_s;
typedef struct mthread_s *mthread_t;

typedef enum {
    ATTR_NONE           = 0,
    ATTR_DETACHED       = (1 << 1),
    ATTR_DETACHED_FREE  = (1 << 2),
} mthread_attr_t;

struct mthread_mutex_s {
    volatile int nb_thread;
    mthread_tst_t lock;
    mthread_list_t *list;   /* lock management already handled */
};
typedef struct mthread_mutex_s mthread_mutex_t;

#define MTHREAD_MUTEX_INITIALIZER { .nb_thread = 0, .lock = 0, .list = NULL }

struct mthread_mutexattr_s;
typedef struct mthread_mutexattr_s mthread_mutexattr_t;

struct mthread_cond_s {
    mthread_tst_t lock;         /* Protects the structure.  */
    mthread_list_t *list;       /* List of waiting threads. */
};
typedef struct mthread_cond_s mthread_cond_t;

struct mthread_condattr_s;
typedef struct mthread_condattr_s mthread_condattr_t;

typedef mthread_tst_t mthread_once_t;

struct mthread_sem_s {
    volatile unsigned int value;    /* Number of wait that are done at the same time. */
    mthread_mutex_t mutex;          /* Protect max_permit.                            */
};
typedef struct mthread_sem_s mthread_sem_t;

/* Function for handling threads.  */

extern int mthread_create(mthread_t *__threadp, const mthread_attr_t __attr, void *(*__start_routine) (void *), void *__arg);

extern mthread_t mthread_self(void);
extern int  mthread_equal (mthread_t __thread1, mthread_t __thread2);
extern void mthread_exit  (void *__retval);
extern int  mthread_join  (mthread_t __th, void **__thread_return);
extern void mthread_yield ();

/* Initialize mthread. */

extern void mthread_init(void);
extern void *mthread_main(void *arg);
